/**
 * Block Notices
 *
 * This file is responsible for rendering Abort Messages from the Table Rate Shipping plugin.
 *
 * @package woocommerce-distance-rate-shipping
 */

import { useSelect } from '@wordpress/data';
import { registerPlugin } from '@wordpress/plugins';
import {
	ExperimentalOrderShippingPackages,
	StoreNotice,
} from '@woocommerce/blocks-checkout';
import { RawHTML } from '@wordpress/element';

/**
 * Create a store notice component.
 *
 * @param notice
 * @param index
 * @param type
 * @returns {JSX.Element}
 */
const createStoreNotice = ( notice, index, type = 'info' ) => {
	if ( 'debug' === type ) {
		type = 'info';
	}

	const message = <RawHTML>{ notice }</RawHTML>;

	return (
		<StoreNotice key={ index } status={ type } isDismissible={ false }>
			{ message }
		</StoreNotice>
	);
};

/**
 * Notices component.
 *
 * @param messages
 *
 * @returns {JSX.Element}
 * @constructor
 */
const Notices = ( { messages } ) => {
	const currentMessage = messages[0] ? messages[0] : null;

	if ( ! currentMessage ) {
		return null;
	}

	return (
		<div className="wc-block-components-shipping-rates-control__package wc-distance-rate-shipping-block-notices">
			{ createStoreNotice( currentMessage, 0, 'warning' ) }
		</div>
	);
};

const render = () => {
	const { abortMessages, hasShippingRates } = useSelect( ( select ) => {
		const storeCartData = select( 'wc/store/cart' ).getCartData();
		const shipments = storeCartData?.shippingRates || [];

		let hasShippingRates = false;
		let abortMessages = [];

		if ( ! storeCartData.extensions || ! storeCartData.extensions[ 'woocommerce_distance_rate_shipping' ] || ! storeCartData.extensions[ 'woocommerce_distance_rate_shipping' ].abort_messages ) {
			return {
				abortMessages,
				hasShippingRates,
			};
		}

		hasShippingRates = shipments.some(
			( shipmentInfo ) => {
				return shipmentInfo.shipping_rates.length > 0 && shipmentInfo.shipping_rates.some(
					( rate ) => ( ! rate.rate_id.startsWith( 'pickup_location:' ) )
				);
			}
		);

		abortMessages =
			storeCartData?.extensions?.[ 'woocommerce_distance_rate_shipping' ]
				?.abort_messages;

		return {
			abortMessages,
			hasShippingRates,
		};
	}, [] );

	// Ensure we only show abort messages if no shipping rates are available.
	if ( hasShippingRates ) {
		return null;
	}

	return (
		<ExperimentalOrderShippingPackages>
			<Notices messages={ abortMessages } />
		</ExperimentalOrderShippingPackages>
	);
};

registerPlugin( 'wc-distance-rate-abort-notices', {
	render,
	scope: 'woocommerce-checkout',
} );
